"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var bson_1 = require("bson");
var AuthEvent_1 = require("../../auth/internal/AuthEvent");
var Base64_1 = require("../../internal/common/Base64");
var Method_1 = __importDefault(require("../../internal/net/Method"));
var StitchAuthDocRequest_1 = require("../../internal/net/StitchAuthDocRequest");
var StitchAuthRequest_1 = require("../../internal/net/StitchAuthRequest");
var RebindEvent_1 = require("./RebindEvent");
var CoreStitchServiceClientImpl = (function () {
    function CoreStitchServiceClientImpl(requestClient, routes, name) {
        this.serviceField = "service";
        this.argumentsField = "arguments";
        this.requestClient = requestClient;
        this.serviceRoutes = routes;
        this.serviceName = name;
        this.serviceBinders = [];
        this.allocatedStreams = [];
    }
    CoreStitchServiceClientImpl.prototype.callFunction = function (name, args, decoder) {
        return this.requestClient.doAuthenticatedRequestWithDecoder(this.getCallServiceFunctionRequest(name, args), decoder);
    };
    CoreStitchServiceClientImpl.prototype.streamFunction = function (name, args, decoder) {
        var _this = this;
        return this.requestClient.openAuthenticatedStreamWithDecoder(this.getStreamServiceFunctionRequest(name, args), decoder).then(function (newStream) {
            _this.allocatedStreams.push(newStream);
            return newStream;
        });
    };
    CoreStitchServiceClientImpl.prototype.bind = function (binder) {
        this.serviceBinders.push(binder);
    };
    CoreStitchServiceClientImpl.prototype.onRebindEvent = function (rebindEvent) {
        switch (rebindEvent.type) {
            case RebindEvent_1.RebindEventType.AUTH_EVENT:
                var authRebindEvent = rebindEvent;
                if (authRebindEvent.event.kind === AuthEvent_1.AuthEventKind.ActiveUserChanged) {
                    this.closeAllocatedStreams();
                }
                break;
            default:
                break;
        }
        this.serviceBinders.forEach(function (binder) {
            binder.onRebindEvent(rebindEvent);
        });
    };
    CoreStitchServiceClientImpl.prototype.getStreamServiceFunctionRequest = function (name, args) {
        var body = { name: name };
        if (this.serviceName !== undefined) {
            body[this.serviceField] = this.serviceName;
        }
        body[this.argumentsField] = args;
        var reqBuilder = new StitchAuthRequest_1.StitchAuthRequest.Builder();
        reqBuilder
            .withMethod(Method_1.default.GET)
            .withPath(this.serviceRoutes.functionCallRoute +
            ("?stitch_request=" + encodeURIComponent(Base64_1.base64Encode(bson_1.EJSON.stringify(body)))));
        return reqBuilder.build();
    };
    CoreStitchServiceClientImpl.prototype.getCallServiceFunctionRequest = function (name, args) {
        var body = { name: name };
        if (this.serviceName !== undefined) {
            body[this.serviceField] = this.serviceName;
        }
        body[this.argumentsField] = args;
        var reqBuilder = new StitchAuthDocRequest_1.StitchAuthDocRequest.Builder();
        reqBuilder
            .withMethod(Method_1.default.POST)
            .withPath(this.serviceRoutes.functionCallRoute);
        reqBuilder.withDocument(body);
        return reqBuilder.build();
    };
    CoreStitchServiceClientImpl.prototype.closeAllocatedStreams = function () {
        this.allocatedStreams.forEach(function (stream) {
            if (stream.isOpen()) {
                stream.close();
            }
        });
        this.allocatedStreams = [];
    };
    return CoreStitchServiceClientImpl;
}());
exports.default = CoreStitchServiceClientImpl;
//# sourceMappingURL=CoreStitchServiceClientImpl.js.map