import { Decoder } from "../../internal/common/Codec";
import { Storage } from "../../internal/common/Storage";
import EventStream from "../../internal/net/EventStream";
import Response from "../../internal/net/Response";
import { StitchAuthRequest } from "../../internal/net/StitchAuthRequest";
import StitchRequestClient from "../../internal/net/StitchRequestClient";
import Stream from "../../Stream";
import StitchCredential from "../StitchCredential";
import { AuthEvent } from "./AuthEvent";
import AuthInfo from "./AuthInfo";
import CoreStitchUser from "./CoreStitchUser";
import StitchAuthRequestClient from "./StitchAuthRequestClient";
import { StitchAuthRoutes } from "./StitchAuthRoutes";
import StitchUserFactory from "./StitchUserFactory";
export default abstract class CoreStitchAuth<TStitchUser extends CoreStitchUser> implements StitchAuthRequestClient {
    readonly authInfo: AuthInfo;
    readonly isLoggedIn: boolean;
    readonly user: TStitchUser | undefined;
    readonly hasDeviceId: boolean;
    protected readonly deviceId: string | undefined;
    protected readonly requestClient: StitchRequestClient;
    protected readonly authRoutes: StitchAuthRoutes;
    protected abstract userFactory: StitchUserFactory<TStitchUser>;
    protected abstract deviceInfo: {
        [key: string]: string;
    };
    private activeUserAuthInfo;
    private readonly storage;
    private currentUser?;
    private allUsersAuthInfo;
    private readonly accessTokenRefresher;
    protected constructor(requestClient: StitchRequestClient, authRoutes: StitchAuthRoutes, storage: Storage, useTokenRefresher?: boolean);
    listUsers(): TStitchUser[];
    doAuthenticatedRequest(stitchReq: StitchAuthRequest, authInfo?: AuthInfo | undefined): Promise<Response>;
    doAuthenticatedRequestWithDecoder<T>(stitchReq: StitchAuthRequest, decoder?: Decoder<T>): Promise<T>;
    openAuthenticatedEventStream(stitchReq: StitchAuthRequest, open?: boolean): Promise<EventStream>;
    openAuthenticatedStreamWithDecoder<T>(stitchReq: StitchAuthRequest, decoder?: Decoder<T>): Promise<Stream<T>>;
    refreshAccessToken(): Promise<void>;
    switchToUserWithId(userId: string): TStitchUser;
    loginWithCredentialInternal(credential: StitchCredential): Promise<TStitchUser>;
    linkUserWithCredentialInternal(user: CoreStitchUser, credential: StitchCredential): Promise<TStitchUser>;
    logoutInternal(): Promise<void>;
    logoutUserWithIdInternal(userId: string): Promise<void>;
    removeUserInternal(): Promise<void>;
    removeUserWithIdInternal(userId: string): Promise<void>;
    close(): void;
    protected abstract onAuthEvent(): any;
    protected abstract dispatchAuthEvent(event: AuthEvent<TStitchUser>): any;
    private prepareAuthRequest;
    private handleAuthFailureForEventStream;
    private handleAuthFailure;
    private tryRefreshAccessToken;
    private prepUser;
    private attachAuthOptions;
    private doLogin;
    private doLoginRequest;
    private processLogin;
    private processLoginResponse;
    private doGetUserProfile;
    private doLogout;
    private clearActiveUserAuth;
    private clearUserAuthTokens;
}
